@@ -374,7 +374,7 @@ class TabularEditor(Editor):
                 self._multi_selected_rows_changed(rows)
 
     def _multi_selected_items_changed(self, event):
-        values = self.values
+        values = self.value
         try:
             added = [ values.index(item) for item in event.added ]
             removed = [ values.index(item) for item in event.removed ]
@@ -487,7 +487,6 @@ class TabularEditor(Editor):
         column, row = self.control.columnAt(pos.x()), self.control.rowAt(pos.y())
         menu = self.adapter.get_menu(self.object, self.name, row, column)
         if menu :
-            qmenu = menu.create_menu( self.control, self )
             self._menu_context = {'selection' : self.object,
                              'object':  self.object,
                              'editor':  self,
@@ -496,6 +495,7 @@ class TabularEditor(Editor):
                              'item':    self.adapter.get_item(self.object, self.name, row),
                              'info':    self.ui.info,
                              'handler': self.ui.handler }
+            qmenu = menu.create_menu( self.control, self )
             qmenu.exec_(self.control.mapToGlobal(pos))
             self._menu_context = None
 
@@ -503,12 +503,12 @@ class TabularEditor(Editor):
         column = self.control.columnAt(pos.x())
         menu = self.adapter.get_column_menu(self.object, self.name, -1, column)
         if menu :
-            qmenu = menu.create_menu( self.control, self )
             self._menu_context = {'selection' : self.object, 'object':  self.object,
                              'editor':  self,
                              'column':  column,
                              'info':    self.ui.info,
                              'handler': self.ui.handler }
+            qmenu = menu.create_menu( self.control, self )
             qmenu.exec_(self.control.mapToGlobal(pos))
             self._menu_context = None
         else:
@@ -594,18 +594,24 @@ class _TableView(QtGui.QTableView):
 
         # Configure the row headings
         vheader = self.verticalHeader()
-        vheader.hide()
+        if factory.show_row_titles:
+            vheader.setHighlightSections(False)
+        else:
+            vheader.hide()
 
-        # Set a default height for rows. Although setting the resize mode to
-        # ResizeToContents would provide the best sizes, this is far too
-        # expensive when the TabularEditor has a large amount of data. Instead,
-        # we make a reasonable guess based on the minimum size hint and the font
-        # of the first row.
-        size = vheader.minimumSectionSize()
-        font = editor.adapter.get_font(editor.object, editor.name, 0)
-        if font is not None:
-            size = max(size, QtGui.QFontMetrics(QtGui.QFont(font)).height())
-        vheader.setDefaultSectionSize(size)
+        if factory.show_row_titles and factory.auto_resize_rows:
+            vheader.setResizeMode(QtGui.QHeaderView.ResizeToContents)
+        else:
+            # Set a default height for rows. Although setting the resize mode to
+            # ResizeToContents would provide the best sizes, this is far too
+            # expensive when the TabularEditor has a large amount of data. Instead,
+            # we make a reasonable guess based on the minimum size hint and the font
+            # of the first row.
+            size = vheader.minimumSectionSize()
+            font = editor.adapter.get_font(editor.object, editor.name, 0)
+            if font is not None:
+                size = max(size, QtGui.QFontMetrics(QtGui.QFont(font)).height())
+            vheader.setDefaultSectionSize(size)
 
         # Configure the column headings.
         hheader = self.horizontalHeader()
@@ -629,7 +635,10 @@ class _TableView(QtGui.QTableView):
 
         # Configure drag and drop behavior
         self.setDragEnabled(True)
-        self.setDragDropMode(QtGui.QAbstractItemView.InternalMove)
+        if factory.editable:
+            self.viewport().setAcceptDrops(True)
+        if factory.drag_move:
+            self.setDragDropMode(QtGui.QAbstractItemView.InternalMove)
         self.setDropIndicatorShown(True)
 
     def keyPressEvent(self, event):
