@@ -26,6 +26,8 @@ from pyface.qt import QtCore, QtGui
 
 from traitsui.ui_traits import SequenceTypes
 
+from .clipboard import PyMimeData
+
 #-------------------------------------------------------------------------------
 #  Constants:
 #-------------------------------------------------------------------------------
@@ -131,6 +133,9 @@ class TabularModel(QtCore.QAbstractTableModel):
         editor = self._editor
         row = mi.row()
         column = mi.column()
+        
+        if not mi.isValid():
+            return QtCore.Qt.ItemIsDropEnabled
 
         flags = QtCore.Qt.ItemIsEnabled
         if editor.factory.selectable:
@@ -139,28 +144,36 @@ class TabularModel(QtCore.QAbstractTableModel):
         # If the adapter defines get_can_edit_cell(), use it to determine
         # editability over the row-wise get_can_edit().
         if (editor.factory.editable and 'edit' in editor.factory.operations and
-            hasattr(editor.adapter, 'get_can_edit_cell')):
+                hasattr(editor.adapter, 'get_can_edit_cell')):
             if editor.adapter.get_can_edit_cell(editor.object, editor.name,
-                row, column):
+                                                row, column):
                 flags |= QtCore.Qt.ItemIsEditable
         elif (editor.factory.editable and 'edit' in editor.factory.operations and
-            editor.adapter.get_can_edit(editor.object, editor.name, row)):
+                editor.adapter.get_can_edit(editor.object, editor.name, row)):
             flags |= QtCore.Qt.ItemIsEditable
 
-        if (editor.factory.editable and 'move' in editor.factory.operations and
-            editor.adapter.get_drag(editor.object, editor.name, row) is not None):
-            flags |= QtCore.Qt.ItemIsDragEnabled | QtCore.Qt.ItemIsDropEnabled
+        if editor.adapter.get_drag(editor.object, editor.name, row) is not None:
+            flags |= QtCore.Qt.ItemIsDragEnabled
+            
+        if editor.factory.editable:
+            flags |=  QtCore.Qt.ItemIsDropEnabled
 
         return flags
 
     def headerData(self, section, orientation, role):
         """ Reimplemented to return the header data.
         """
-        if orientation != QtCore.Qt.Horizontal or role != QtCore.Qt.DisplayRole:
+        if role != QtCore.Qt.DisplayRole:
             return None
 
         editor = self._editor
-        label = editor.adapter.get_label(section, editor.object)
+
+        label = None
+        if orientation == QtCore.Qt.Vertical:
+            label = editor.adapter.get_row_label(section, editor.object)
+        elif orientation == QtCore.Qt.Horizontal:
+            label = editor.adapter.get_label(section, editor.object)
+
         return label
 
     def rowCount(self, mi):
@@ -224,16 +237,20 @@ class TabularModel(QtCore.QAbstractTableModel):
         """ Reimplemented to expose our internal MIME type for drag and drop
             operations.
         """
-        return [ tabular_mime_type ]
+        return [ tabular_mime_type, PyMimeData.MIME_TYPE,
+                PyMimeData.NOPICKLE_MIME_TYPE ]
 
     def mimeData(self, indexes):
         """ Reimplemented to generate MIME data containing the rows of the
             current selection.
         """
-        mime_data = QtCore.QMimeData()
-        rows = list(set([ index.row() for index in indexes ]))
-        data = QtCore.QByteArray(str(rows[0]))
-        for row in rows[1:]:
+        rows = sorted(set([ index.row() for index in indexes ]))
+        items = [self._editor.adapter.get_drag(
+                self._editor.object, self._editor.name, row)
+                for row in rows]
+        mime_data = PyMimeData.coerce(items)
+        data = QtCore.QByteArray(str(id(self)))
+        for row in rows:
             data.append(' %i' % row)
         mime_data.setData(tabular_mime_type, data)
         return mime_data
@@ -244,13 +261,38 @@ class TabularModel(QtCore.QAbstractTableModel):
         if action == QtCore.Qt.IgnoreAction:
             return False
 
+        # this is a drag from a tabular model
         data = mime_data.data(tabular_mime_type)
-        if data.isNull():
-            return False
-
-        current_rows = map(int, str(data).split(' '))
-        self.moveRows(current_rows, parent.row())
-        return True
+        if not data.isNull() and action == QtCore.Qt.MoveAction:
+            id_and_rows = map(int, str(data).split(' '))
+            table_id = id_and_rows[0]
+            # is it from ourself?
+            if table_id == id(self):
+                current_rows = id_and_rows[1:]
+                self.moveRows(current_rows, parent.row())
+                return True
+        
+        # this is an external drag
+        data = PyMimeData.coerce(mime_data).instance()
+        if data is not None:
+            if not isinstance(data, list):
+                data = [data]
+            editor = self._editor
+            object = editor.object
+            name = editor.name
+            adapter = editor.adapter
+            if row == -1 and parent.isValid():
+                # find correct row number
+                row = parent.row()
+            if row == -1 and adapter.len(object, name) == 0:
+                # if empty list, target is after end of list
+                row = 0
+            if all(adapter.get_can_drop(object, name, row, item)
+                                    for item in data):
+                for item in reversed(data):
+                    self.dropItem(item, row)
+                return True
+        return False
 
     def supportedDropActions(self):
         """ Reimplemented to allow items to be moved.
@@ -261,6 +303,19 @@ class TabularModel(QtCore.QAbstractTableModel):
     #  TabularModel interface:
     #---------------------------------------------------------------------------
 
+    def dropItem(self, item, row):
+        """ Handle a Python object being dropped onto a row """
+        editor = self._editor
+        object = editor.object
+        name = editor.name
+        adapter = editor.adapter
+        destination = adapter.get_dropped(object, name, row, item)
+        
+        if destination == 'after':
+            row += 1
+        
+        adapter.insert(object, name, row, item)
+
     def moveRow(self, old_row, new_row):
         """ Convenience method to move a single row.
         """
