@@ -57,7 +57,7 @@ class _CallAfter(QtCore.QObject):
     # The mutex around the list of pending calls.
     _calls_mutex = QtCore.QMutex()
 
-    def __init__(self, handler, *args):
+    def __init__(self, handler, *args, **kwds):
         """ Initialise the call.
         """
         QtCore.QObject.__init__(self)
@@ -65,6 +65,7 @@ class _CallAfter(QtCore.QObject):
         # Save the details of the call.
         self._handler = handler
         self._args = args
+        self._kwds = kwds
 
         # Add this to the list.
         self._calls_mutex.lock()
@@ -85,7 +86,7 @@ class _CallAfter(QtCore.QObject):
         """
         if event.type() == _QT_TRAITS_EVENT:
             # Invoke the handler
-            self._handler(*self._args)
+            self._handler(*self._args, **self._kwds)
 
             # We cannot remove from self._calls here. QObjects don't like being
             # garbage collected during event handlers (there are tracebacks,
@@ -103,11 +104,11 @@ class _CallAfter(QtCore.QObject):
         del self._calls[self._calls.index(self)]
         self._calls_mutex.unlock()
 
-def ui_handler ( handler, *args ):
+def ui_handler ( handler, *args, **kwds ):
     """ Handles UI notification handler requests that occur on a thread other
         than the UI thread.
     """
-    _CallAfter(handler, *args)
+    _CallAfter(handler, *args, **kwds)
 
 # Tell the traits notification handlers to use this UI handler
 set_ui_handler( ui_handler )
@@ -228,8 +229,11 @@ class GUIToolkit ( Toolkit ):
             pdx = screen_dx
             pdy = screen_dy
         else:
-            px = parent.x()
-            py = parent.y()
+            pos = parent.pos()
+            if int(parent.windowFlags()) & QtCore.Qt.Window == 0 :
+                pos = parent.mapToGlobal(pos)
+            px = pos.x()
+            py = pos.y()
             pdx = parent.width()
             pdy = parent.height()
 
@@ -368,13 +372,18 @@ class GUIToolkit ( Toolkit ):
             return
 
         elif events == 'keys':
-            class KeyEventCatcher(QtCore.QObject):
+            class KeyEventHook(QtCore.QObject):
                 def eventFilter(self, object, event):
                     if event.type() == QtCore.QEvent.KeyPress:
                         return handler(event)
                     else:
                         return QtCore.QObject.eventFilter(self, object, event)
-            control.installEventFilter(KeyEventCatcher(control))
+
+            # It's unsafe to parent the event filter with the object it's
+            # filtering, so we store a reference to it here to ensure that it's
+            # not garbage collected prematurely.
+            ui._key_event_hook = KeyEventHook()
+            control.installEventFilter(ui._key_event_hook)
 
     #---------------------------------------------------------------------------
     #  Indicates that an event should continue to be processed by the toolkit
@@ -448,10 +457,7 @@ class GUIToolkit ( Toolkit ):
               specific color format.
         """
         return {
-            # fixme: This should be updated to have the Qt window background
-            # color. This is just a placeholder to prevent traits.ui.api
-            # problems.
-            'WindowColor': 0xFFFFFF
+            'WindowColor': QtGui.QApplication.palette().color(QtGui.QPalette.Window),
         }
 
     #---------------------------------------------------------------------------
@@ -518,6 +524,10 @@ class GUIToolkit ( Toolkit ):
         import compound_editor as ce
         return ce.ToolkitEditorFactory( *args, **traits )
 
+    def styled_date_editor ( self, *args, **traits ):
+        import styled_date_editor as sde
+        return sde.ToolkitEditorFactory( *args, **traits )
+
     # Custom:
     def custom_editor ( self, *args, **traits ):
         import custom_editor as ce
