@@ -42,6 +42,9 @@ from helper \
 # List of all predefined system button names:
 SystemButtons = ['Undo', 'Redo', 'Apply', 'Revert', 'OK', 'Cancel', 'Help']
 
+# List of alternative context items that might handle an Action 'perform':
+PerformHandlers = ( 'object', 'model' )
+
 def default_icon():
     from pyface.image_resource import ImageResource
     return ImageResource('frame.png')
@@ -139,7 +142,23 @@ class BasePanel(object):
         if method is not None:
             method( self.ui.info )
         else:
-            action.perform()
+            # TODO extract to common superclass for wx and qt4
+
+            # cf. commit cdf76eb5965c0184114c4674e06b28beee04af36
+            # (July 28, 2008, dmorrill) that fixes this issue for the
+            # wx backend
+
+            # look for the method in the context of the handler
+            context = self.ui.context
+            for item in PerformHandlers:
+                handler = context.get( item, None )
+                if handler is not None:
+                    method = getattr( handler, action.action, None )
+                    if method is not None:
+                        method()
+                        break
+            else:
+                action.perform()
 
     #---------------------------------------------------------------------------
     #  Check to see if a specified 'system' button is in the buttons list, and
@@ -296,6 +315,9 @@ class _StickyDialog(QtGui.QDialog):
         self._ui = ui
         self._result = None
 
+    def showEvent(self, e):
+        self.raise_()
+
     def closeEvent(self, e):
         """Reimplemented to check when the clicks the window close button.
         (Note that QDialog doesn't get a close event when the dialog is closed
