@@ -32,7 +32,7 @@ import wx
 # Make sure a wx.App object is created early:
 _app = wx.GetApp()
 if _app is None:
-    _app = wx.PySimpleApp()
+    _app = wx.App(redirect=False)
 
 from traits.api \
     import HasPrivateTraits, Instance, Property, Category, cached_property
@@ -461,6 +461,7 @@ class GUIToolkit ( Toolkit ):
     def destroy_control ( self, control ):
         """ Destroys a specified GUI toolkit control.
         """
+        _popEventHandlers(control)
         control.Destroy()
 
     #---------------------------------------------------------------------------
@@ -471,6 +472,8 @@ class GUIToolkit ( Toolkit ):
         """ Destroys all of the child controls of a specified GUI toolkit
             control.
         """
+        for child in control.GetChildren():
+            _popEventHandlers(child)
         control.DestroyChildren()
 
     #---------------------------------------------------------------------------
@@ -776,3 +779,13 @@ class WXDockWindowTheme ( Category, DockWindowTheme ):
 
         return image.create_image().ConvertToBitmap()
 
+
+#-------------------------------------------------------------------------------
+def _popEventHandlers(ctrl):
+    """ Pop any event handlers that have been pushed on to a window and its
+        children.
+    """
+    while ctrl is not ctrl.GetEventHandler():
+        ctrl.PopEventHandler(True)
+    for child in ctrl.GetChildren():
+        _popEventHandlers(child)
